/*
 *  file:     m392demo.c
 *  revision: 1.1
 *  date:     09/05/00
 *  author:   JG
 * ----------------------------------------------------------------------
 *
 * M392 APIS based demo application.
 *
 * This application is part of the M392 ANSI-C library and demonstrates
 * the usage of an M392 on an APIS supported platform. It is possible
 * to execute this application using interrupts or polling the RDY bit.
 *
 * This demo application uses APIS functions and M392 functions
 * provided by m392lib.c.
 *
 * ----------------------------------------------------------------------
 * Copyright 2000 by AcQuisition Technology bv (c)
 * All Rights Reserved
 * Reproduced Under License
 *
 * This source code is the proprietary confidential property of
 * AcQuisition Technology bv, and is provided to the licensee
 * for documentation and educational purposes only. Reproduction,
 * publication, or any form of distribution to any party other than
 * the licensee is strictly prohibited.
 * ----------------------------------------------------------------------
 * Edition History
 *
 * #    date        Comments                                        by
 * ---  --------    ----------------------------------------------  ---
 * 1.0  03/01/00    First release                                   JG
 * 1.1  09/05/00    apis_open call changed                          JG
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "../../../APIS/SOFTWARE/COMMON/DEFS/apis.h"
#include "../../../MMODID/SOFTWARE/LIB/ideeprom.h"
#include "../LIB/m392defs.h"

/* Forward Declarations of local functions
 */
void usage(char *pname);

/*
 * Function:    main
 *
 * Description: M392/APIS Demo program entry
 *
 * Parameters:  int argc
 *                  number of program arguments
 *              char *argv[]
 *                  pointer to program argument list
 *
 * Returns:     0
 */
int main (int argc, char *argv[])
{
    APIS_HANDLE handle;         /* APIS Handle */
    UINT32 pathid = 0;          /* M392 base address */
    int i, j;                   /* General idici */
    int result;                 /* Command result */
    UINT16 params[16];          /* Array to store parameters */
    IDCODE idcode;              /* ID EEPROM structure */
    UINT16 data;
    int quit=FALSE;
    int use_int=TRUE;

    printf("\nM392 Demo\n");
    printf("by AcQuisition Technology bv 1999\n\n");

    for (i = 1; i < argc; i++) {
        if (argv[i][0] == '-') {
            for (j = 1; argv[i][j]; j++) {
                switch (tolower(argv[i][j])) {
                    case 'b':
                       if (argv[i][++j] == '=')
                           j++;
                       sscanf(argv[i]+j, "%lx", &pathid);
                       while(argv[i][j])
                           j++;
                       j--;
                       break;
                   case 'i':
                       use_int = FALSE;
                       break;
                   default:
                       usage(argv[0]);
                       exit(0);
                }
            }
        }
    }

    /*
     * Open a hardware path to the M392
     */
    if ((result = apis_open(pathid, &handle, 0x100, A08D16)) != 0) {
        printf("Could not open path: 0x%04x\n", result);
        exit(0);
    }

    /*
     * Read and verify contents of the Identification EEPROM
     */
    apis_read(handle, sizeof(UINT16), CTRLREG, &data);
    if (!(data & RESET))
        apis_write(handle, sizeof(UINT16), CTRLREG, (UINT16)(RESET));

    if (get_modid(handle, (void *)&idcode) != 0) {
        printf("ID EEPROM error\n");
        apis_close(handle);
        exit(0);
    }
    if (idcode.id.synccode != IDSYNC) {
        printf("Invalid identification code\n");
        apis_close(handle);
        exit(0);
    }
    printf("Module ID: M%d\n", idcode.id.modnum);
    if (idcode.id.modnum != MODTYPE) {
        printf("Error: wrong module\n");
        apis_close(handle);
        exit(0);
    }

    /*
     * Booting the module
     */
    printf("Booting the module\n");
    if (m392_boot(handle) != 0) {
        printf("Failed\n\n");
        apis_close(handle);
        exit(0);
    }
    printf("Done\n");

    /* Check whether or not the DSP is running */
    if ((result = m392_cmd(handle, SYNC, (UINT16*)(&params), 0, 0))
        != 0) {
        printf("SYNC command failed: 0x%04x\n", result);
        apis_close(handle);
        exit(0);
    }

    /*
     * Get firmware version
     */
    if ((result = m392_cmd(handle, VERSION, (UINT16*)(&params), 0, 4))
        != 0) {
        printf("VERSION command failed: 0x%04x\n", result);
        apis_close(handle);
        exit(0);
    }
    printf("Firmware version: %d.%d\n", params[1]/10, params[1]%10);
    printf("%d-bit ADC M-module\n", params[3]);

    /*
     * Setup input ranges of channels
     */
    params[0] = V10BI;      /* Channel 0 +/- 10 V bipolar */
    params[1] = V5BI;       /* Channel 1 +/- 5 V bipolar */
    params[2] = V10UNI;     /* Channel 2 10 V unipolar */
    params[3] = V5UNI;      /* Channel 3 5 V unipolar */
    params[4] = DISCHAN;    /* Channel 4 disabled */
    params[5] = V10BI;      /* Channel 5 +/- 10 V bipolar */
    params[6] = V5BI;       /* Channel 6 +/- 5 V bipolar */
    params[7] = V10UNI;     /* Channel 7 10 V unipolar */
    params[8] = V5UNI;      /* Channel 8 5 V unipolar */
    params[9] = DISCHAN;    /* Channel 9 disabled */
    params[10] = V10BI;     /* Channel 10 +/- 10 V bipolar */
    params[11] = V5BI;      /* Channel 11 +/- 5 V bipolar */
    params[12] = V10UNI;    /* Channel 12 10 V unipolar */
    params[13] = V5UNI;     /* Channel 13 5 V unipolar */
    params[14] = DISCHAN;   /* Channel 14 disabled */
    params[15] = V10BI;     /* Channel 15 +/- 10 V bipolar */

    printf("\nSetup input ranges of channel 0-15\n");
    if ((result = m392_setupchan(handle, (UINT16*)(&params))) != NOERR) {
        printf("SETUP command failed: 0x%04x\n", result);
        apis_close(handle);
        exit(0);
    }

    /*
     * Setup acquisition time
     */
    printf("Set acquisition time to 1000 microseconds\n");
    /* Set acquisition time to 1000 microseconds */
    if ((result = m392_setupacqtime(handle, TIME1000us)) != NOERR) {
        printf("SETACQTIME command failed: 0x%04x\n", result);
        apis_close(handle);
        exit(0);
    }

    /*
     * Configure moving average filter
     */
    printf("Set MA filter behavior to 64 samples\n");
    /* Set filter behavior to 64 samples */
    if ((result = m392_configmafilter(handle, SAMPLE64)) != NOERR) {
        printf("MAFILTER command failed: 0x%04x\n", result);
        apis_close(handle);
        exit(0);
    }

    if ( use_int == TRUE) {
        /*
         * Unmask interrupt and install interrupt handler
         */
        if (m392_install_irq(handle) != NOERR) {
            printf("Error install interrupt handler\n");
            apis_close(handle);
            exit(0);
        }

        printf("\nScan channel 0-15 after 100 measurement cycles\n");
        i=0;
        while(quit == FALSE) {
            /* Wait for interrupt */
            if ((apis_waitforirq()) == APIS_ESIG)
                quit = TRUE;

            if ((++i%100) == 0) {       /* After 100 measurement cycles */
                for (j=0; j<16; j++) {  /* Read all channels */
                    if ((j%4) == 0)
                        printf("\n");
                    if (( m392_readchannel( handle, (UINT16)j, &data))
                        != 0) {
                        printf("Error reading channel %d\n", j);
                        apis_close(handle);
                        exit(0);
                    }
                    printf("CH%02d 0x%04x\t", j, data);
                }
                printf("\n\n");
            }
        }

        /*
         * Mask interrupt and remove interrupt handler
         */
        m392_remove_irq(handle);
    } else {
        printf("\nScan channel 0-15 after 100 measurement cycles\n");

        for(i=0;i<1000;) {
            m392_wait4rdybit(handle);
            m392_clear_rdybit(handle);
            i++;

            if ((i%100) == 0) {         /* After 100 measurement cycles */
                for (j=0; j<16; j++) {  /* Read all channels */
                    if ((j%4) == 0)
                        printf("\n");
                    if (( m392_readchannel( handle, (UINT16)j, &data))
                        != 0) {
                        printf("Error reading channel %d\n", j);
                        apis_close( handle);
                        exit(0);
                    }
                    printf("CH%02d 0x%04x\t", j, data);
                }
                printf("\n\n");
            }
        }
    }

    /* Close hardware path */
    apis_close(handle);

    printf("M392 Demo is closed by user.\n");
    return 0;
}

/*
 * Function:    usage
 *
 * Description: The program usage is displayed and the program
 *              is terminated
 *
 * Parameters:  pointer to the program name
 *
 * Returns:     nothing
 */
void usage(char *pname)
{
    printf("Syntax: %s [<opts>]\n", pname);
    printf("Options (default):\n");
    printf("  -b=<base>     module base address in hex (0)\n");
    printf("  -i            execute without using interrupts\n");
}
