/*
 *  file:     modideep.c
 *  revision: 1.6
 *  date:     11/05/04
 *  author:   JG
 * ----------------------------------------------------------------------
 *
 * Routines to read the M-module ID EEPROM of a M-module placed on
 * an APIS supported M-module carrier board.
 *
 * ----------------------------------------------------------------------
 * Copyright 1999-2004 by AcQuisition Technology B.V. (c)
 * All Rights Reserved
 * Reproduced Under License
 *
 * This source code is the proprietary confidential property of
 * AcQuisition Technology B.V., and is provided to the licensee
 * for documentation and educational purposes only.  Reproduction,
 * publication, or any form of distribution to any party other than
 * the licensee is strictly prohibited.
 * ----------------------------------------------------------------------
 * Edition History
 *
 * #   date      Comments                                        by
 * --  --------  ----------------------------------------------  ---
 * 1.0 22-04-99  initial version                                 sp
 * 1.1 27-07-99  ported to APIS
 * 1.2 03-05-00  Added id_delay() function to improve execution
 *               speed because apis_delay() takes to much time   jg
 * 1.3 05-09-00  id_delay function changed: dummy1 removed       jg
 * 1.4 26-06-02  id_reset function added and called from
 *               get_modid to put the EEPROM in a defined state.
 *               Function store_modid() and id_write() added.    sp
 * 1.5 11-12-03  Clear control register explicitly               sp
 * 1.6 11-05-04  Condition to id_delay() added for use with      sp
 *               USB to M-module bridge
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "apis.h"
#include "ideeprom.h"

/* Function prototyping */

int get_modid (APIS_HANDLE, void *);                /* Get EEPROM contents */
int store_modid (APIS_HANDLE, void *);              /* Write data to EEPROM */

/* Function prototyping of local functions */

static int id_cmd(APIS_HANDLE,UINT16);              /* EEPROM command cycle */
static int id_rddata(APIS_HANDLE,UINT16 *);         /* EEPROM input cycle */
static int id_read(APIS_HANDLE,UINT16,UINT16*);     /* EEPROM read data word */
static int id_wrdata(APIS_HANDLE, UINT16);          /* EEPROM output cycle */
static int id_write(APIS_HANDLE, UINT16, UINT16);   /* write word to EEPROM */
static void id_delay(APIS_HANDLE);                  /* Insert a delay */
static void id_busy(APIS_HANDLE handle);            /* EEPROM wait until done */
static void id_reset(APIS_HANDLE);                  /* Reset EEPROM */

/* EEPROM definitions */

#define EEPROM      0x00fe  /* Eeprom register offset for word accesses */
#define CS          0x04    /* Chip Select */
#define SK          0x02    /* Serial data clock */
#define DI          0x01    /* Serial data input */
#define DO          0x01    /* Serial data output */
#define BITMASK     0x0100  /* Bit mask */
#define WDS         0x0100  /* write disable */
#define WEN         0x0130  /* write enable */
#define IDWRITE     0x0140  /* writes data */
#define IDREAD      0x0180  /* Reads data stored in memory */


/*
 * get_modid
 *
 * Descripion:  This function reads the contents of the ID EEPROM on
 *              an M-module.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              void *ideep
 *                  pointer to ID eeprom buffer
 *
 * Returns:     0 if successfull else -1
 */
int get_modid (APIS_HANDLE handle, void *ideep)
{
    UINT16 *dpr, reg, data;

    dpr = (UINT16 *)ideep;

    id_reset(handle);       /* reset communication */

    for (reg = 0; reg < EEPSZ; reg++) {
        if (id_read(handle, reg, &data) != 0)
            return -1;
        dpr[reg] = data;
    }
    return 0;
}

/*
 * store_modid
 *
 * Descripion:  This function stores data in the ID EEPROM on
 *              an M-module.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              void *ideep
 *                  pointer to ID eeprom buffer
 *
 * Returns:     0 if successfull else -1
 */
int store_modid (APIS_HANDLE handle, void *ideep)
{
    UINT16 *dpr, reg;

    dpr = (UINT16 *)ideep;  /* initialize pointer */

    id_reset(handle);       /* reset communication */

    for (reg = 0; reg < 16; reg++) {
        if (id_write(handle, reg, *dpr++) != 0)
            return -1;
    }
    return 0;
}


/* Local functions */

/*
 * id_cmd
 *
 * Description: Execute an ID EEPROM command cycle.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 cmds
 *                  command
 *
 * Returns:     0
 */
static int id_cmd(APIS_HANDLE handle, UINT16 cmds)
{
    UINT8 idctrl;
    UINT16 i;

    idctrl = CS;
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);

    for (i = 0; i < 9; i++)
    {
        if (cmds & BITMASK)
            idctrl |= DI;
        else
            idctrl &= ~DI;
        cmds <<= 1;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
        idctrl |= SK;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
        idctrl &= ~SK;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
    }
    return 0;
}

/*
 * id_write - write word to M-module ID EEPROM
 *
 * Descripion:  This function will write a word to the ID EEPROM
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 reg
 *                  register to read
 *              UINT16 data
 *                  data to write
 *
 * Returns:     0 if successfull else -1
 */
static int id_write(APIS_HANDLE handle, UINT16 reg, UINT16 data)
{
    UINT16 temp_data;       /* temporary variable */

    /* write enable */
    id_cmd(handle, (UINT16)WEN);
    apis_write(handle, sizeof(UINT16), EEPROM, 0);

    /* write word */
    id_cmd(handle, (UINT16)(IDWRITE | reg));
    id_wrdata (handle, data);

    /* write disable */
    id_cmd(handle, (UINT16)WDS);
    apis_write(handle, sizeof(UINT16), EEPROM, 0);

    /* verify contents */
    id_cmd(handle, (UINT16)(IDREAD | reg));
    id_rddata (handle, &temp_data);

    if (temp_data != data)
        return -1;

    return 0;
}

/*
 * id_wrdata
 *
 * descripion:  This function will write a data word to the serial
 *              identicication EEPROM, using bit-toggling.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 data
 *                  data to write
 *
 * Returns:     0 if successfull else -1
 */
static int id_wrdata(APIS_HANDLE handle, UINT16 data)
{
    UINT16 idctrl;
    UINT16 i;

    /* chip slect high */
    idctrl = CS;

    for (i=0; i<16; i++) {
        if (data & 0x8000)
            /* data bit is 1 */
            idctrl |= DI;
        else
            /* data bit is 0 */
            idctrl &= ~DI;

        /* shift data bit */
        data <<= 1;

        /* write EEPROM register */
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);

        /* delay */
        id_delay(handle);

        /* clock high */
        idctrl |= SK;

        /* write EEPROM register */
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);

        /* delay */
        id_delay(handle);

        /* clock low */
        idctrl &= ~SK;

        /* write EEPROM register */
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);

        /* delay */
        id_delay(handle);
    }

    /* write 0 to EEPROM register */
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)0);

    /* delay */
    id_busy(handle);

    return 0;
}

/*
 * id_rddata
 *
 * descripion:  This function will read a data word from the serial
 *              identicication EEPROM, using bit-toggling.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 *pData
 *                  pointer to data storage
 *
 * Returns:     0 if successfull else -1
 */
static int id_rddata(APIS_HANDLE handle, UINT16 *pData)
{
    UINT8 idctrl;
    UINT16 data, i, iword;

    do
        apis_read(handle, sizeof (UINT16), EEPROM, (UINT16 *)&iword);
    while (iword & DO);

    data = 0;
    for (i = 0; i < 16; i++)
    {
        idctrl = CS | SK;

        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);

        data <<= 1;

        apis_read(handle, sizeof (UINT16), EEPROM, (UINT16 *)&iword);
        if (iword & DO)
            data++;

        idctrl &= ~SK;

        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);
    }
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)0);
    id_delay(handle);

    *pData = data;

    return 0;
}

/*
 * id_read - read word from M-module ID EEPROM
 *
 * Descripion:  This function will write a read-command to the serial
 *              identicication EEPROM and will read and pass the word.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *              UINT16 reg
 *                  register to read
 *              UINT16 *pData
 *                  pointer to data storage
 *
 * Returns:     0 if successfull else -1
 */
static int id_read(APIS_HANDLE handle, UINT16 reg, UINT16 *pData)
{
    id_cmd(handle, (UINT16)(IDREAD | reg));
    return id_rddata (handle, pData);
}

/*
 * id_reset
 *
 * description: Put EEPROM communication in a defined state
 *              by generating sufficient clock cycles to make
 *              sure no data is pending.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 */
static void id_reset(APIS_HANDLE handle)
{
    UINT8 idctrl;
    UINT16 i;

    /*
     * Clear EEPROM register
     */
    idctrl = 0;
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);

    for (i = 0; i < 64; i++)
    {
        idctrl = CS | SK;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);

        idctrl = CS;
        apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
        id_delay(handle);

    }
    idctrl = 0;
    apis_write(handle, sizeof (UINT16), EEPROM, (UINT16)idctrl);
    id_delay(handle);
}


/*
 * id_delay
 *
 * description: This function will wait a minimum of 250 nanoseconds.
 *              Using dummy reads.
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 */
static void id_delay(APIS_HANDLE handle)
{
#if !USB_MMOD_BRIDGE
    int i;
    UINT16 dummy;

    for (i=0; i<10; i++) {
        apis_read(handle, sizeof(UINT16), EEPROM, &dummy);
    }
#endif
}

/*
 * id_busy
 *
 * description: Wait for write actionto complete
 *
 * Parameters:  APIS_HANDLE handle
 *                  hardware path handle
 *
 * Returns:     Nothing
 */
static void id_busy(APIS_HANDLE handle)
{
    UINT16 idctrl;

    idctrl = CS;

    apis_write(handle, sizeof(UINT16), EEPROM, idctrl);

    do {
        id_delay(handle);
        apis_read(handle, sizeof(UINT16), EEPROM, &idctrl);
    } while (!(idctrl & DO));

    apis_write(handle, sizeof(UINT16), EEPROM, 0);
}

